import time
from dataclasses import asdict
from pathlib import Path
from typing import Optional, Union
from urllib import request

from loguru import logger

from netspresso.clients.auth import TokenHandler, auth_client
from netspresso.clients.auth.response_body import UserResponse
from netspresso.clients.launcher import launcher_client_v2
from netspresso.clients.launcher.v2.schemas import InputLayer, ResponseConvertTaskItem
from netspresso.clients.launcher.v2.schemas.task.convert.response_body import (
    ConvertTask,
)
from netspresso.enums import (
    DataType,
    DeviceName,
    Framework,
    ServiceCredit,
    SoftwareVersion,
    Status,
    TaskStatusForDisplay,
)
from netspresso.metadata.converter import ConverterMetadata
from netspresso.utils import FileHandler, check_credit_balance
from netspresso.utils.metadata import MetadataHandler


class ConverterV2:
    def __init__(self, token_handler: TokenHandler, user_info: UserResponse):
        """Initialize the Converter."""

        self.token_handler = token_handler
        self.user_info = user_info

    def _download_converted_model(
        self, convert_task: ConvertTask, local_path: str
    ) -> None:
        """Download the converted model with given conversion task or conversion task uuid.

        Args:
            conversion_task (ConvertTask): Launcher Model Object or the uuid of the conversion task.

        Raises:
            e: If an error occurs while getting the conversion task information.
        """

        try:
            if convert_task.status == TaskStatusForDisplay.ERROR:
                raise FileNotFoundError(
                    "The conversion is Failed. There is no file available for download."
                )
            if convert_task.status != TaskStatusForDisplay.FINISHED:
                raise FileNotFoundError(
                    "The conversion is in progress. There is no file available for download at the moment."
                )

            download_url = launcher_client_v2.converter.download_model_file(
                convert_task_uuid=convert_task.convert_task_id,
                access_token=self.token_handler.tokens.access_token,
            ).data.presigned_download_url

            request.urlretrieve(download_url, local_path)
            logger.info(f"Model downloaded at {Path(local_path)}")

        except Exception as e:
            logger.error(f"Download converted model failed. Error: {e}")
            raise e

    def convert_model(
        self,
        input_model_path: str,
        output_dir: str,
        target_framework: Union[str, Framework],
        target_device_name: Union[str, DeviceName],
        target_data_type: Union[str, DataType] = DataType.FP16,
        target_software_version: Optional[Union[str, SoftwareVersion]] = None,
        input_layer: Optional[InputLayer] = None,
        dataset_path: Optional[str] = None,
        wait_until_done: bool = True,
    ) -> ConvertTask:
        """Convert a model to the specified framework.

        Args:
            input_model_path (str): The file path where the model is located.
            output_dir (str): The local folder path to save the converted model.
            target_framework (Union[str, Framework]): The target framework name.
            target_device_name (Union[str, DeviceName]): Target device name. Required if target_device is not specified.
            target_data_type (Union[str, DataType]): Data type of the model. Default is DataType.FP16.
            target_software_version (Union[str, SoftwareVersion], optional): Target software version.
                Required if target_device_name is one of the Jetson devices.
            input_layer (InputShape, optional): Target input shape for conversion (e.g., dynamic batch to static batch).
            dataset_path (str, optional): Path to the dataset. Useful for certain conversions.
            wait_until_done (bool): If True, wait for the conversion result before returning the function.
                                If False, request the conversion and return  the function immediately.

        Raises:
            e: If an error occurs during the model conversion.

        Returns:
            ResponseConvertTaskItem: Model conversion task result.
        """

        FileHandler.check_input_model_path(input_model_path)

        self.token_handler.validate_token()

        default_model_path, extension = FileHandler.get_path_and_extension(
            folder_path=output_dir, framework=target_framework
        )
        FileHandler.create_unique_folder(folder_path=output_dir)
        converter_metadata = ConverterMetadata()
        MetadataHandler.save_json(
            data=asdict(converter_metadata), folder_path=output_dir
        )

        try:
            current_credit = auth_client.get_credit(
                self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )
            check_credit_balance(
                user_credit=current_credit, service_credit=ServiceCredit.MODEL_CONVERT
            )

            # GET presigned_model_upload_url
            presigned_url_response = (
                launcher_client_v2.converter.presigned_model_upload_url(
                    access_token=self.token_handler.tokens.access_token,
                    input_model_path=input_model_path,
                )
            )

            # UPLOAD model_file
            launcher_client_v2.converter.upload_model_file(
                access_token=self.token_handler.tokens.access_token,
                input_model_path=input_model_path,
                presigned_upload_url=presigned_url_response.data.presigned_upload_url,
            )

            # VALIDATE model_file
            validate_model_response = launcher_client_v2.converter.validate_model_file(
                access_token=self.token_handler.tokens.access_token,
                input_model_path=input_model_path,
                ai_model_id=presigned_url_response.data.ai_model_id,
            )

            # START convert task
            response = launcher_client_v2.converter.start_task(
                access_token=self.token_handler.tokens.access_token,
                input_model_id=presigned_url_response.data.ai_model_id,
                target_device_name=target_device_name,
                target_framework=target_framework,
                data_type=target_data_type,
                input_layer=input_layer,
                software_version=target_software_version,
                dataset_path=dataset_path,
            )

            if wait_until_done:
                while True:
                    # Poll Convert Task status
                    response = launcher_client_v2.converter.read_task(
                        access_token=self.token_handler.tokens.access_token,
                        task_id=response.data.convert_task_id,
                    )
                    if response.data.status in [
                        TaskStatusForDisplay.FINISHED,
                        TaskStatusForDisplay.ERROR,
                        TaskStatusForDisplay.TIMEOUT,
                    ]:
                        break
                    time.sleep(3)

            self._download_converted_model(
                convert_task=response.data,
                local_path=str(default_model_path.with_suffix(extension)),
            )

            if launcher_client_v2.is_cloud():
                remaining_credit = auth_client.get_credit(
                    self.token_handler.tokens.access_token,
                    self.token_handler.verify_ssl,
                )
                logger.info(
                    f"{ServiceCredit.MODEL_BENCHMARK} credits have been consumed. Remaining Credit: {remaining_credit}"
                )

            convert_task = response.data

            input_model_info = validate_model_response.data

            task_options = launcher_client_v2.converter.read_model_task_options(
                access_token=self.token_handler.tokens.access_token,
                ai_model_id=convert_task.input_model_id,
            ).data

            if convert_task.status == TaskStatusForDisplay.FINISHED:
                converter_metadata.status = Status.COMPLETED
                logger.info("Convert task successfully completed.")
            else:
                converter_metadata.status = Status.ERROR
                logger.info("Convert task failed with an error.")

            converter_metadata.input_model_path = input_model_path
            converter_metadata.converted_model_path = output_dir
            converter_metadata.model_info = input_model_info.to()
            converter_metadata.convert_task_info = convert_task.to(
                input_model_info.uploaded_file_name
            )
            for task_option in task_options:
                converter_metadata.available_options.append(task_option.to())

            MetadataHandler.save_json(
                data=asdict(converter_metadata), folder_path=output_dir
            )

            return response.data

        except Exception as e:
            logger.error(f"Convert failed. Error: {e}")
            converter_metadata.status = Status.ERROR
            MetadataHandler.save_json(
                data=asdict(converter_metadata), folder_path=output_dir
            )
            raise e

        except KeyboardInterrupt:
            converter_metadata.status = Status.STOPPED
            MetadataHandler.save_json(
                data=asdict(converter_metadata), folder_path=output_dir
            )

    def get_conversion_task(self, conversion_task_id: str) -> ConvertTask:
        """Get the conversion task information with given conversion task uuid.

        Args:
            conversion_task_id (str): Convert task UUID of the convert task.

        Raises:
            e: If an error occurs during the model conversion.

        Returns:
            ConversionTask: Model conversion task dictionary.
        """

        self.token_handler.validate_token()

        response = launcher_client_v2.converter.read_task(
            access_token=self.token_handler.tokens.access_token,
            task_id=conversion_task_id,
        )
        return response.data



from enum import Enum, IntEnum


class ServiceCredit(IntEnum):
    ADVANCED_COMPRESSION = 50
    AUTOMATIC_COMPRESSION = 25
    MODEL_CONVERT = 50
    MODEL_BENCHMARK = 25


class MembershipType(str, Enum):
    BASIC = "BASIC"
    PRO = "PRO"
    PREMIUM = "PREMIUM"



from .auth import CreditResponse, LoginResponse, TokenRequest, UserInfo

__all__ = [
    "LoginRequest",
    "LoginResponse",
    "CreditResponse",
    "UserInfo",
    "TokenRequest",
]

from ...request_body import LoginRequest



from pydantic import BaseModel, EmailStr, Field

from netspresso.clients.auth.response_body import (
    CreditResponse,
    TokenResponse,
    UserDetailResponse,
    UserResponse,
)


class TokenRequest(BaseModel):
    access_token: str = Field(..., description="Access Token")
    refresh_token: str = Field(..., description="Refresh Token")


class LoginResponse(BaseModel):
    current_time: str = Field(..., description="Login Time")
    region: str = Field(..., description="User Resion")
    tokens: TokenResponse = Field(
        default_factory=TokenResponse, description="Session Token"
    )


class UserInfo(BaseModel):
    user_id: str = Field(..., description="User ID")
    email: EmailStr = Field(..., description="Email")
    username: str = Field(..., description="User Name")
    detail_data: UserDetailResponse = Field(..., description="User Detail")
    is_active: bool = Field(..., description="Active Status")
    is_admin: bool = Field(..., description="Admin Status")
    current_time: str = Field(..., description="Current Time")
    created_time: str = Field(..., description="Created Time")
    last_login_time: str = Field(..., description="Last Login Time")
    free: int = Field(0, description="Free Credit")
    reward: int = Field(0, description="Reward Credit")
    contract: int = Field(0, description="Contract Credit")
    paid: int = Field(0, description="Paid Credit")
    total: int = Field(0, description="Total Credit")

    def to(self) -> UserResponse:
        return UserResponse(
            **{
                "user_id": self.user_id,
                "email": self.email,
                "detail_data": self.detail_data,
                "credit_info": CreditResponse(
                    **{
                        "free": self.free,
                        "reward": self.reward,
                        "contract": self.contract,
                        "paid": self.paid,
                        "total": self.total,
                    }
                ),
            }
        )



import sys
from pathlib import Path
from typing import Dict, List, Optional
from urllib import request

from loguru import logger

from netspresso.clients.auth import TokenHandler, auth_client
from netspresso.clients.compressor import compressor_client_v2
from netspresso.clients.compressor.v2.schemas import (
    ModelBase,
    Options,
    RecommendationOptions,
    RequestAutomaticCompressionParams,
    RequestAvailableLayers,
    RequestCreateCompression,
    RequestCreateModel,
    RequestCreateRecommendation,
    RequestUpdateCompression,
    RequestUploadModel,
    RequestValidateModel,
    ResponseCompression,
    ResponseCompressionItem,
    ResponseSelectMethod,
    UploadFile,
)
from netspresso.clients.launcher import launcher_client_v2
from netspresso.compressor.utils.file import read_file_bytes
from netspresso.compressor.utils.onnx import export_onnx
from netspresso.enums import CompressionMethod, Framework, Module, RecommendationMethod, ServiceCredit, Status, TaskType
from netspresso.metadata.compressor import CompressorMetadata
from netspresso.utils import FileHandler
from netspresso.utils.metadata import MetadataHandler


class CompressorV2:
    def __init__(self, token_handler: TokenHandler) -> None:
        """Initialize the Compressor."""

        self.token_handler = token_handler

    def check_credit_balance(self, service_credit):
        current_credit = auth_client.get_credit(
            access_token=self.token_handler.tokens.access_token,
            verify_ssl=self.token_handler.verify_ssl
        )
        service_name = service_credit.name.replace("_", " ").lower()
        if current_credit < service_credit:
            sys.exit(f"Your current balance of {current_credit} credits is insufficient to complete the task. \n{service_credit} credits are required for one {service_name} task. \nFor additional credit, please contact us at netspresso@nota.ai.")

    def print_remaining_credit(self):
        if compressor_client_v2.is_cloud():
            remaining_credit = auth_client.get_credit(
                self.token_handler.tokens.access_token, verify_ssl=self.token_handler.verify_ssl
            )
            logger.info(
                f"{ServiceCredit.ADVANCED_COMPRESSION} credits have been consumed. Remaining Credit: {remaining_credit}"
            )

    def create_metadata(self, folder_path) -> CompressorMetadata:
        metadata = CompressorMetadata()
        MetadataHandler.save_json(data=metadata.asdict(), folder_path=folder_path)

        return metadata

    def _get_available_options(self, compressed_model_info, default_model_path: str):
        if compressed_model_info.detail.framework in [Framework.PYTORCH, Framework.ONNX]:
            export_onnx(default_model_path, [compressed_model_info.detail.input_layer])
            options_response = launcher_client_v2.converter.read_framework_options(
                access_token=self.token_handler.tokens.access_token,
                framework=Framework.ONNX,
            )
        else:
            options_response = launcher_client_v2.converter.read_framework_options(
                access_token=self.token_handler.tokens.access_token,
                framework=Framework.TENSORFLOW_KERAS,
            )

        available_options = options_response.data

        return available_options

    def upload_model(
        self,
        input_model_path: str,
        input_shapes: List[Dict[str, int]] = None,
        framework: Framework = Framework.PYTORCH,
    ) -> ModelBase:
        self.token_handler.validate_token()

        try:
            logger.info("Uploading Model...")

            FileHandler.check_input_model_path(input_model_path)

            object_name = Path(input_model_path).name

            create_model_request = RequestCreateModel(object_name=object_name)
            create_model_response = compressor_client_v2.create_model(
                request_data=create_model_request,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )

            file_content = read_file_bytes(file_path=input_model_path)
            upload_model_request = RequestUploadModel(url=create_model_response.data.presigned_url)
            file = UploadFile(file_name=object_name, file_content=file_content)
            upload_model_response = compressor_client_v2.upload_model(
                request_data=upload_model_request,
                file=file,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )

            if not upload_model_response:
                # TODO: Confirm upload success
                raise Exception("Upload model failed.")

            validate_model_request = RequestValidateModel(framework=framework, input_layers=input_shapes)
            validate_model_response = compressor_client_v2.validate_model(
                ai_model_id=create_model_response.data.ai_model_id,
                request_data=validate_model_request,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )

            model_info = validate_model_response.data

            logger.info(f"Upload model successfully. Model ID: {model_info.ai_model_id}")

            return model_info

        except Exception as e:
            logger.error(f"Upload model failed. Error: {e}")
            raise e

    def get_model(self, model_id: str) -> ModelBase:
        self.token_handler.validate_token()

        try:
            logger.info("Getting model...")
            read_model_response = compressor_client_v2.read_model(
                ai_model_id=model_id,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )
            model_info = read_model_response.data

            logger.info("Get model successfully.")

            return model_info

        except Exception as e:
            logger.error(f"Get model failed. Error: {e}")
            raise e

    def download_model(self, model_id: str, local_path: str) -> None:
        self.token_handler.validate_token()

        try:
            logger.info("Downloading model...")
            download_link = compressor_client_v2.download_model(
                ai_model_id=model_id,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )
            request.urlretrieve(download_link.data.presigned_url, local_path)
            logger.info(f"Model downloaded at {Path(local_path)}")

        except Exception as e:
            logger.error(f"Download model failed. Error: {e}")
            raise e

    def select_compression_method(
        self,
        model_id: str,
        compression_method: CompressionMethod,
        options: Optional[Options] = Options(),
    ) -> ResponseSelectMethod:
        self.token_handler.validate_token()

        try:
            logger.info("Selecting compression method...")

            get_available_layers_request = RequestAvailableLayers(
                compression_method=compression_method,
                options=options,
            )
            get_available_layers_response = compressor_client_v2.get_available_layers(
                ai_model_id=model_id,
                request_data=get_available_layers_request,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )

            available_layers_info = get_available_layers_response.data

            logger.info("Select compression method successfully.")

            return available_layers_info

        except Exception as e:
            logger.error(f"Select compression method failed. Error: {e}")
            raise e

    def get_compression(self, compression_id: str) -> ResponseCompression:
        self.token_handler.validate_token()

        try:
            logger.info("Getting compression...")
            read_compression_response = compressor_client_v2.read_compression(
                compression_id=compression_id,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )

            compression_info = read_compression_response.data

            logger.info("Get compression successfully.")

            return compression_info

        except Exception as e:
            logger.error(f"Get compression failed. Error: {e}")
            raise e

    def upload_dataset(self, compression_id: str, dataset_path: str) -> None:
        self.token_handler.validate_token()

        try:
            logger.info("Uploading dataset...")
            file_content = read_file_bytes(file_path=dataset_path)
            object_name = Path(dataset_path).name
            file = UploadFile(file_name=object_name, file_content=file_content)
            compressor_client_v2.upload_dataset(
                compression_id=compression_id,
                file=file,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )
            logger.info("Upload dataset successfully.")

        except Exception as e:
            logger.error(f"Upload dataset failed. Error: {e}")
            raise e

    def compress_model(
        self,
        compression: ResponseSelectMethod,
        output_dir: str,
        dataset_path: Optional[str] = None,
    ) -> Dict:
        self.token_handler.validate_token()

        try:
            logger.info("Compressing model...")

            model_info = self.get_model(compression.input_model_id)

            output_dir = FileHandler.create_unique_folder(folder_path=output_dir)
            metadata = self.create_metadata(folder_path=output_dir)
            default_model_path, extension = FileHandler.get_path_and_extension(
                folder_path=output_dir, framework=model_info.detail.framework
            )

            self.check_credit_balance(service_credit=ServiceCredit.ADVANCED_COMPRESSION)

            create_compression_request = RequestCreateCompression(
                ai_model_id=compression.input_model_id,
                compression_method=compression.compression_method,
                options=compression.options,
            )
            create_compression_response = compressor_client_v2.create_compression(
                request_data=create_compression_request,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl
            )

            for available_layers in compression.available_layers:
                if available_layers.values:
                    available_layers.use = True

            if dataset_path and compression.compression_method == CompressionMethod.PR_NN:
                self.upload_dataset(compression_id=compression.compression_id, dataset_path=dataset_path)

            update_compression_request = RequestUpdateCompression(
                available_layers=compression.available_layers,
                options=compression.options,
            )
            update_compression_response = compressor_client_v2.compress_model(
                compression_id=create_compression_response.data.compression_id,
                request_data=update_compression_request,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl
            )

            compression_info = update_compression_response.data

            self.download_model(
                model_id=compression_info.input_model_id,
                local_path=default_model_path.with_suffix(extension),
            )

            compressed_model_info = self.get_model(model_id=compression_info.input_model_id)
            available_options = self._get_available_options(compressed_model_info, default_model_path)

            logger.info(f"Compress model successfully. Compressed Model ID: {compression_info.input_model_id}")

            self.print_remaining_credit()

            if compressed_model_info.detail.framework in [Framework.PYTORCH, Framework.ONNX]:
                metadata.update_compressed_onnx_model_path(
                    compressed_onnx_model_path=default_model_path.with_suffix(".onnx").as_posix()
                )
            metadata.update_compressed_model_path(
                compressed_model_path=default_model_path.with_suffix(extension).as_posix()
            )
            metadata.update_model_info(framework=model_info.detail.framework, input_shapes=[model_info.detail.input_layer])
            metadata.update_compression_info(
                method=compression.compression_method,
                options=compression.options,
                layers=compression.available_layers,
            )
            metadata.update_results(model=model_info, compressed_model=compressed_model_info)
            metadata.update_status(status=Status.COMPLETED)
            metadata.update_available_options(available_options)

            MetadataHandler.save_json(data=metadata.asdict(), folder_path=output_dir)

            return compression_info

        except Exception as e:
            logger.error(f"Compress model failed. Error: {e}")
            metadata.update_status(status=Status.ERROR)
            MetadataHandler.save_json(data=metadata.asdict(), folder_path=output_dir)
            raise e

        except KeyboardInterrupt:
            logger.error("Compress model stopped.")
            metadata.update_status(status=Status.STOPPED)
            MetadataHandler.save_json(data=metadata.asdict(), folder_path=output_dir)

    def recommendation_compression(
        self,
        compression_method: CompressionMethod,
        recommendation_method: RecommendationMethod,
        recommendation_ratio: float,
        input_model_path: str,
        output_dir: str,
        input_shapes: List[Dict[str, int]],
        framework: Framework = Framework.PYTORCH,
        options: RecommendationOptions = RecommendationOptions(),
        dataset_path: Optional[str] = None,
    ) -> Dict:
        self.token_handler.validate_token()

        try:
            logger.info("Compressing recommendation-based model...")

            output_dir = FileHandler.create_unique_folder(folder_path=output_dir)
            metadata = self.create_metadata(folder_path=output_dir)

            default_model_path, extension = FileHandler.get_path_and_extension(
                folder_path=output_dir, framework=framework
            )

            self.check_credit_balance(service_credit=ServiceCredit.ADVANCED_COMPRESSION)

            model_info = self.upload_model(
                framework=framework,
                input_model_path=input_model_path,
                input_shapes=input_shapes,
            )

            create_compression_request = RequestCreateCompression(
                ai_model_id=model_info.ai_model_id,
                compression_method=compression_method,
                options=options,
            )
            create_compression_response = compressor_client_v2.create_compression(
                request_data=create_compression_request,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl
            )

            if dataset_path and compression_method == CompressionMethod.PR_NN:
                self.upload_dataset(
                    compression_id=create_compression_response.data.compression_id, dataset_path=dataset_path
                )

            logger.info("Calculating recommendation values...")
            create_recommendation_request = RequestCreateRecommendation(
                recommendation_method=recommendation_method,
                recommendation_ratio=recommendation_ratio,
                options=options,
            )
            create_recommendation_response = compressor_client_v2.create_recommendation(
                compression_id=create_compression_response.data.compression_id,
                request_data=create_recommendation_request,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )

            logger.info("Compressing model...")
            update_compression_request = RequestUpdateCompression(
                available_layers=create_recommendation_response.data.available_layers,
                options=options,
            )
            update_compression_response = compressor_client_v2.compress_model(
                compression_id=create_compression_response.data.compression_id,
                request_data=update_compression_request,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl
            )

            compression_info = update_compression_response.data

            self.download_model(
                model_id=compression_info.input_model_id,
                local_path=default_model_path.with_suffix(extension),
            )

            compressed_model_info = self.get_model(model_id=compression_info.input_model_id)
            available_options = self._get_available_options(compressed_model_info, default_model_path)

            logger.info(f"Recommendation compression successfully. Compressed Model ID: {compression_info.input_model_id}")

            self.print_remaining_credit()

            if compressed_model_info.detail.framework in [Framework.PYTORCH, Framework.ONNX]:
                metadata.update_compressed_onnx_model_path(
                    compressed_onnx_model_path=default_model_path.with_suffix(".onnx").as_posix()
                )
            metadata.update_compressed_model_path(
                compressed_model_path=default_model_path.with_suffix(extension).as_posix()
            )
            metadata.update_model_info(framework=framework, input_shapes=input_shapes)
            metadata.update_compression_info(
                method=compression_info.compression_method,
                ratio=recommendation_ratio,
                options=compression_info.options,
                layers=compression_info.available_layers,
            )
            metadata.update_results(model=model_info, compressed_model=compressed_model_info)
            metadata.update_status(status=Status.COMPLETED)
            metadata.update_available_options(available_options)

            MetadataHandler.save_json(data=metadata.asdict(), folder_path=output_dir)

            return compression_info

        except Exception as e:
            logger.error(f"Recommendation compression failed. Error: {e}")
            metadata.update_status(status=Status.ERROR)
            MetadataHandler.save_json(data=metadata.asdict(), folder_path=output_dir)
            raise e

        except KeyboardInterrupt:
            logger.error("Recommendation compression stopped.")
            metadata.update_status(status=Status.STOPPED)
            MetadataHandler.save_json(data=metadata.asdict(), folder_path=output_dir)

    def automatic_compression(
        self,
        input_model_path: str,
        output_dir: str,
        input_shapes: List[Dict[str, int]],
        framework: Framework = Framework.PYTORCH,
        compression_ratio: float = 0.5,
    ) -> Dict:
        self.token_handler.validate_token()

        try:
            logger.info("Compressing automatic-based model...")

            output_dir = FileHandler.create_unique_folder(folder_path=output_dir)
            metadata = self.create_metadata(folder_path=output_dir)

            default_model_path, extension = FileHandler.get_path_and_extension(
                folder_path=output_dir, framework=framework
            )

            self.check_credit_balance(service_credit=ServiceCredit.AUTOMATIC_COMPRESSION)

            model_info = self.upload_model(framework=framework, input_model_path=input_model_path, input_shapes=input_shapes)

            logger.info("Compressing model...")
            automatic_compression_request = RequestAutomaticCompressionParams(compression_ratio=compression_ratio)
            automatic_compression_response = compressor_client_v2.compress_model_with_automatic(
                ai_model_id=model_info.ai_model_id,
                request_data=automatic_compression_request,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )

            compression_info = automatic_compression_response.data

            self.download_model(
                model_id=compression_info.input_model_id,
                local_path=default_model_path.with_suffix(extension),
            )

            compressed_model_info = self.get_model(model_id=compression_info.input_model_id)
            available_options = self._get_available_options(compressed_model_info, default_model_path)

            logger.info(f"Automatic compression successfully. Compressed Model ID: {compression_info.input_model_id}")

            self.print_remaining_credit()

            if compressed_model_info.detail.framework in [Framework.PYTORCH, Framework.ONNX]:
                metadata.update_compressed_onnx_model_path(default_model_path.with_suffix(".onnx").as_posix())
            metadata.update_compressed_model_path(default_model_path.with_suffix(extension).as_posix())
            metadata.update_model_info(framework=framework, input_shapes=input_shapes)
            metadata.update_compression_info(
                method=compression_info.compression_method,
                ratio=compression_ratio,
                options=compression_info.options,
                layers=compression_info.available_layers,
            )
            metadata.update_results(model=model_info, compressed_model=compressed_model_info)
            metadata.update_status(status=Status.COMPLETED)
            metadata.update_available_options(available_options)

            MetadataHandler.save_json(data=metadata.asdict(), folder_path=output_dir)

            return compression_info

        except Exception as e:
            logger.error(f"Automatic compression failed. Error: {e}")
            metadata.update_status(status=Status.ERROR)
            MetadataHandler.save_json(data=metadata.asdict(), folder_path=output_dir)
            raise e

        except KeyboardInterrupt:
            logger.error("Automatic compression stopped.")
            metadata.update_status(status=Status.STOPPED)
            MetadataHandler.save_json(data=metadata.asdict(), folder_path=output_dir)



import time
from dataclasses import asdict
from pathlib import Path
from typing import Dict, Optional, Union

from loguru import logger

from netspresso.clients.auth import TokenHandler, auth_client
from netspresso.clients.auth.response_body import UserResponse
from netspresso.clients.launcher import launcher_client_v2
from netspresso.clients.launcher.v2.schemas.task.benchmark.response_body import (
    BenchmarkTask,
)
from netspresso.enums import Status, TaskStatusForDisplay, TaskType
from netspresso.enums.credit import ServiceCredit
from netspresso.enums.device import (
    DeviceName,
    HardwareType,
    SoftwareVersion,
)
from netspresso.metadata.benchmarker import BenchmarkerMetadata
from netspresso.utils import FileHandler, check_credit_balance
from netspresso.utils.metadata import MetadataHandler


class BenchmarkerV2:
    def __init__(self, token_handler: TokenHandler, user_info: UserResponse) -> None:
        """Initialize the Benchmarker."""

        self.token_handler = token_handler
        self.user_info = user_info

    def get_benchmark_task(self, benchmark_task_id: str) -> BenchmarkTask:
        """Get information about the specified benchmark task using the benchmark task UUID.

        Args:
            benchmark_task_id (str): Benchmark task UUID of the benchmark task.

        Raises:
            e: If an error occurs while retrieving information about the benchmark task.

        Returns:
            BenchmarkTask: Model benchmark task object.
        """

        self.token_handler.validate_token()

        response = launcher_client_v2.benchmarker.read_task(
            access_token=self.token_handler.tokens.access_token,
            task_id=benchmark_task_id,
        )
        return response.data

    def benchmark_model(
        self,
        input_model_path: str,
        target_device_name: DeviceName,
        target_software_version: Optional[Union[str, SoftwareVersion]] = None,
        target_hardware_type: Optional[Union[str, HardwareType]] = None,
        wait_until_done: bool = True,
    ) -> BenchmarkTask:
        """Benchmark the specified model on the specified device.

        Args:
            input_model_path (str): The file path where the model is located.
            target_device_name (DeviceName): Target device name.
            target_software_version (Union[str, SoftwareVersion], optional): Target software version. Required if target_device_name is one of the Jetson devices.
            target_hardware_type (Union[str, HardwareType], optional): Hardware type. Acceleration options for processing the model inference.
            wait_until_done (bool): If True, wait for the conversion result before returning the function.
                                If False, request the conversion and return the function immediately.

        Raises:
            e: If an error occurs during the benchmarking of the model.

        Returns:
            Dict: Model benchmark task dictionary.
        """

        FileHandler.check_input_model_path(input_model_path)

        self.token_handler.validate_token()

        file_name = "benchmark"

        try:
            folder_path = Path(input_model_path).parent

            benchmarker_metadata = BenchmarkerMetadata()
            metadatas = []

            file_path = folder_path / f"{file_name}.json"

            if FileHandler.check_exists(file_path):
                metadatas = MetadataHandler.load_json(file_path)
                # metadatas.append(asdict(benchmarker_metadata))

            current_credit = auth_client.get_credit(
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )
            check_credit_balance(
                user_credit=current_credit, service_credit=ServiceCredit.MODEL_BENCHMARK
            )

            # GET presigned_model_upload_url
            presigned_url_response = (
                launcher_client_v2.benchmarker.presigned_model_upload_url(
                    access_token=self.token_handler.tokens.access_token,
                    input_model_path=input_model_path,
                )
            )

            # UPLOAD model_file
            launcher_client_v2.benchmarker.upload_model_file(
                access_token=self.token_handler.tokens.access_token,
                input_model_path=input_model_path,
                presigned_upload_url=presigned_url_response.data.presigned_upload_url,
            )

            # VALIDATE model_file
            validate_model_response = (
                launcher_client_v2.benchmarker.validate_model_file(
                    access_token=self.token_handler.tokens.access_token,
                    input_model_path=input_model_path,
                    ai_model_id=presigned_url_response.data.ai_model_id,
                )
            )

            # START convert task
            response = launcher_client_v2.benchmarker.start_task(
                access_token=self.token_handler.tokens.access_token,
                input_model_id=presigned_url_response.data.ai_model_id,
                data_type=validate_model_response.data.detail.data_type,
                target_device_name=target_device_name,
                hardware_type=target_hardware_type,
                input_layer=validate_model_response.data.detail.input_layer,
                software_version=target_software_version,
            )

            if wait_until_done:
                while True:
                    # Poll Benchmark Task status
                    response = launcher_client_v2.benchmarker.read_task(
                        access_token=self.token_handler.tokens.access_token,
                        task_id=response.data.benchmark_task_id,
                    )
                    if response.data.status in [
                        TaskStatusForDisplay.FINISHED,
                        TaskStatusForDisplay.ERROR,
                        TaskStatusForDisplay.TIMEOUT,
                    ]:
                        break
                    time.sleep(3)

            if launcher_client_v2.is_cloud():
                remaining_credit = auth_client.get_credit(
                    self.token_handler.tokens.access_token,
                    self.token_handler.verify_ssl,
                )
                logger.info(
                    f"{ServiceCredit.MODEL_BENCHMARK} credits have been consumed. Remaining Credit: {remaining_credit}"
                )

            benchmark_task = response.data
            input_model_info = validate_model_response.data

            if benchmark_task.status == TaskStatusForDisplay.FINISHED:
                benchmarker_metadata.status = Status.COMPLETED
                logger.info("Benchmark task successfully completed.")
            else:
                benchmarker_metadata.status = Status.ERROR
                logger.info("Benchmark task failed with an error.")

            benchmarker_metadata.task_type = TaskType.BENCHMARK
            benchmarker_metadata.input_model_path = input_model_path
            benchmarker_metadata.benchmark_task_info = benchmark_task.to()
            benchmarker_metadata.convert_task_info = benchmark_task.benchmark_result.to(
                file_size=input_model_info.file_size_in_mb
            )

            metadatas.append(asdict(benchmarker_metadata))
            MetadataHandler.save_json(
                data=metadatas,
                folder_path=folder_path,
                file_name=file_name,
            )

            return response.data

        except Exception as e:
            logger.error(f"Benchmark failed. Error: {e}")
            benchmarker_metadata.status = Status.ERROR
            metadatas.append(asdict(benchmarker_metadata))
            MetadataHandler.save_json(
                data=metadatas,
                folder_path=folder_path,
                file_name=file_name,
            )
            raise e

        except KeyboardInterrupt:
            benchmarker_metadata.status = Status.STOPPED
            metadatas.append(asdict(benchmarker_metadata))
            MetadataHandler.save_json(
                data=metadatas,
                folder_path=folder_path,
                file_name=file_name,
            )



def read_file_bytes(file_path: str) -> bytes:
    with open(file_path, "rb") as f:
        file_byte = f.read()
    return file_byte


def make_file_form_data_object(file_name, file_content):
    return [("file", (file_name, file_content))]



from dataclasses import dataclass, field
from typing import List

from netspresso.enums.device import DeviceName, HardwareType, SoftwareVersion
from netspresso.enums.model import (
    DataType,
    Framework,
)


@dataclass
class InputShape:
    batch: int = 1
    channel: int = 3
    dimension: List[int] = field(default_factory=list)


@dataclass
class ModelInfo:
    data_type: DataType = ""
    framework: Framework = ""
    input_shapes: List[InputShape] = field(default_factory=list)


@dataclass
class SoftwareVersions:
    software_version: SoftwareVersion = ""
    display_software_versions: str = ""


@dataclass
class DeviceInfo:
    device_name: DeviceName = ""
    display_device_name: str = ""
    display_brand_name: str = ""
    software_versions: SoftwareVersions = field(default_factory=SoftwareVersions)
    data_types: List[DataType] = field(default_factory=list)
    hardware_types: List[HardwareType] = field(default_factory=list)


@dataclass
class AvailableOption:
    framework: Framework = ""
    display_framework: str = ""
    devices: List[DeviceInfo] = field(default_factory=list)



from netspresso.clients.compressor.v1.schemas.compression import Options

from .compression import (
    CompressionMethod,
    GroupPolicy,
    LayerNorm,
    Policy,
    RecommendationMethod,
    StepOp,
)
from .credit import MembershipType, ServiceCredit
from .device import (
    DeviceName,
    DisplaySoftwareVersion,
    HardwareType,
    SoftwareVersion,
    TaskStatus,
)
from .metadata import Status, TaskType
from .model import DataType, Extension, Framework, OriginFrom
from .module import Module
from .tao.action import ConvertAction, ExperimentAction
from .task import LauncherTask, Task, TaskStatusForDisplay

__all__ = [
    "ServiceCredit",
    "TaskType",
    "Status",
    "CompressionMethod",
    "RecommendationMethod",
    "Policy",
    "GroupPolicy",
    "LayerNorm",
    "Task",
    "Framework",
    "Framework",
    "Extension",
    "OriginFrom",
    "DataType",
    "DeviceName",
    "SoftwareVersion",
    "DisplaySoftwareVersion",
    "HardwareType",
    "TaskStatus",
    "Module",
    "Options",
    "ConvertAction",
    "ExperimentAction",
    "StepOp",
    "MembershipType",
    "DisplaySoftwareVersion",
    "LauncherTask",
    "TaskStatusForDisplay",
]



import json
from dataclasses import asdict

import requests
from loguru import logger

from netspresso.clients.auth.response_body import TokenResponse, UserResponse
from netspresso.clients.auth.v1.schemas import (
    LoginRequest,
    LoginResponse,
    TokenRequest,
    UserInfo,
)
from netspresso.clients.config import Config
from netspresso.clients.utils import get_headers


class AuthClientV1:
    def __init__(self, config: Config):
        """Initialize the UserSession.

        Args:
            email (str): The email address for a user account.
            password (str): The password for a user account.
        """

        self.config = config
        self.host = self.config.HOST
        self.port = self.config.PORT
        self.uri_prefix = self.config.URI_PREFIX
        self.base_url = f"{self.host}:{self.port}{self.uri_prefix}"

    def login(self, email, password, verify_ssl: bool = True) -> TokenResponse:
        try:
            url = f"{self.base_url}/auth/local/login"
            data = LoginRequest(username=email, password=password)
            response = requests.post(
                url, json=asdict(data), headers=get_headers(), verify=verify_ssl
            )
            response_body = json.loads(response.text)

            if response.status_code == 200 or response.status_code == 201:
                session = LoginResponse(**response_body)
                logger.info("Login successfully")
                return session.tokens
            else:
                raise Exception(response_body["detail"])

        except Exception as e:
            logger.error(f"Login failed. Error: {e}")
            raise e

    def get_user_info(self, access_token, verify_ssl: bool = True) -> UserResponse:
        try:
            url = f"{self.base_url}/user"
            response = requests.get(
                url, headers=get_headers(access_token=access_token), verify=verify_ssl
            )
            response_body = json.loads(response.text)

            if response.status_code == 200 or response.status_code == 201:
                user_info = UserInfo(**response_body)
                logger.info("Successfully got user information")
                return user_info.to()
            else:
                raise Exception(response_body["detail"])

        except Exception as e:
            logger.error(f"Failed to get user information. Error: {e}")
            raise e

    def get_credit(self, access_token, verify_ssl: bool = True) -> int:
        try:
            user_info = self.get_user_info(access_token, verify_ssl)
            logger.info("Successfully got user credit")
            return user_info.credit_info.total
        except Exception as e:
            logger.error(f"Failed to get user credit. Error: {e}")
            raise e

    def reissue_token(
        self, access_token, refresh_token, verify_ssl: bool = True
    ) -> TokenResponse:
        try:
            url = f"{self.base_url}/auth/token"
            data = TokenRequest(access_token=access_token, refresh_token=refresh_token)
            response = requests.post(
                url,
                data=data.json(),
                headers=get_headers(json_type=True),
                verify=verify_ssl,
            )
            response_body = json.loads(response.text)

            if response.status_code == 200 or response.status_code == 201:
                token_response = TokenResponse(**response_body["tokens"])
                logger.info("Successfully reissued token")
                return token_response
            else:
                raise Exception(response_body["detail"])

        except Exception as e:
            logger.info(f"Failed to reissue token. Error: {e}")
            raise e

